@@ -5,25 +5,21 @@ Created at 2022.05.31
 
 This script contains all the command line parameters
 """
-import gzip
 import os
 import sys
 from multiprocessing import cpu_count
 from typing import Optional, Dict, Set, Tuple
 
 import click
-import matplotlib as mpl
 from click_option_group import optgroup
 from loguru import logger
 
 from trackplot.base.GenomicLoci import GenomicLoci
 from trackplot.conf.config import CLUSTERING_METHOD, COLORS, COLORMAP, DISTANCE_METRIC, IMAGE_TYPE, NORMALIZATION
 from trackplot.file.ATAC import ATAC
-from trackplot.plot import Plot
-
-__version__ = "0.1.9"
-__author__ = "ygidtu & Ran Zhou"
-__email__ = "ygidtu@gmail.com"
+from trackplot.plot import Plot, __version__
+from trackplot.plot_func import load_barcodes
+from trackplot.server import run, __PLOT__
 
 
 def decode_region(region: str):
@@ -50,7 +46,6 @@ class FileList(object):
                  trans: Optional[str] = None,
                  depth: Optional[int] = None,
                  tad: Optional[str] = None):
-
         self.path = os.path.abspath(os.path.expanduser(path))
 
         if not os.path.exists(self.path):
@@ -75,39 +70,6 @@ class FileList(object):
                f"color: {self.color} \ncategory: {self.category} \nlibrary: {self.library}"
 
 
-def load_barcodes(barcode: str) -> Tuple[Dict[str, Dict[str, Set[str]]], Dict[str, str]]:
-    u"""
-    as name says
-    :param barcode: the path to barcode file
-    """
-    r = gzip.open(barcode, "rt") if barcode.endswith(".gz") else open(barcode, "r")
-    res = {}
-    colors = {}
-    for line in r:
-        line = line.strip().split()
-        if len(line) >= 4:
-            key, bc, group, color = line[:4]
-            colors[group] = color
-        elif len(line) == 3:
-            key, bc, group = line[:3]
-        elif len(line) == 2:
-            key, bc = line
-            group = ""
-        else:
-            continue
-
-        if key not in res.keys():
-            res[key] = {}
-
-        if group not in res[key]:
-            res[key][group] = set()
-
-        res[key][group].add(bc)
-
-    r.close()
-    return res, colors
-
-
 def __read_iter__(path):
     with open(path) as r:
         for idx, line in enumerate(r):
@@ -131,7 +93,7 @@ def process_file_list(infile: str, category: str = "density"):
             for idx, line in __read_iter__(infile):
                 path, category = line[0], line[1]
 
-                if category not in ["bam", "bigwig", "bw", "depth", "igv", "atac"]:
+                if category not in ["bam", "bigwig", "bw", "depth", "igv", "atac", "bedgraph", "bg"]:
                     raise ValueError(f"{category} is not supported in density plot.")
 
                 if len(line) < 3:
@@ -150,7 +112,7 @@ def process_file_list(infile: str, category: str = "density"):
             for idx, line in __read_iter__(infile):
                 path, category = line[0], line[1]
 
-                if category not in ["bam", "bigwig", "bw", "depth", "atac"]:
+                if category not in ["bam", "bigwig", "bw", "depth", "atac", "bedgraph", "bg"]:
                     raise ValueError(f"{category} is not supported in heatmap plot.")
 
                 if len(line) < 3:
@@ -172,7 +134,7 @@ def process_file_list(infile: str, category: str = "density"):
             for idx, line in __read_iter__(infile):
                 path, category = line[0], line[1]
 
-                if category not in ["bam", "bigwig", "bw", "depth"]:
+                if category not in ["bam", "bigwig", "bw", "depth", "bedgraph", "bg"]:
                     raise ValueError(f"{category} is not supported in density plot.")
 
                 if len(line) < 3:
@@ -254,8 +216,9 @@ def process_file_list(infile: str, category: str = "density"):
 
 @click.command(context_settings=dict(help_option_names=['-h', '--help']), no_args_is_help=True)
 @click.version_option(__version__, message="Current version %(version)s")
-@click.option("--debug", is_flag=True, help="enable debug level log")
-@click.option("-e", "--event", type=click.STRING, required=True,
+@click.option("--verbose", is_flag=True, default=False, help="enable debug level log")
+@click.option("--logfile", type=click.Path(), help="save log info into file")
+@click.option("-e", "--event", type=click.STRING,
               help="Event range eg: chr1:100-200:+")
 @optgroup.group("Common input files configuration")
 @optgroup.option("--color-factor", default=1, type=click.IntRange(min=1),
@@ -292,7 +255,7 @@ def process_file_list(infile: str, category: str = "density"):
                  help="The width of output file, default adjust image width by content", show_default=True)
 @optgroup.option("--backend", type=click.STRING, default="Agg", help="Recommended backend", show_default=True)
 @optgroup.group("Reference settings")
-@optgroup.option("-r", "--reference", type=click.Path(exists=True),
+@optgroup.option("-r", "--annotation", type=click.Path(exists=True),
                  help="Path to gtf file, both transcript and exon tags are necessary")
 @optgroup.option("--interval", type=click.Path(exists=True),
                  help="Path to list of interval files in bed format, "
@@ -302,7 +265,7 @@ def process_file_list(infile: str, category: str = "density"):
 @optgroup.option("--no-gene", is_flag=True, type=click.BOOL, show_default=True,
                  help="Do not show gene id next to transcript id")
 @optgroup.option("--domain", default=False, is_flag=True, type=click.BOOL, show_default=True,
-                 help="Add domain information into reference track")
+                 help="Add domain information into annotation track")
 @optgroup.option("--proxy", default=None, type=click.STRING,
                  help="The http or https proxy for EBI/Uniprot requests,"
                       "if `--domain` is True, eg: http://127.0.0.1:1080")
@@ -310,12 +273,12 @@ def process_file_list(infile: str, category: str = "density"):
                  show_default=True,
                  help="The requests timeout when `--domain` is True.")
 @optgroup.option("--local-domain", default="", is_flag=False, type=click.STRING, show_default=True,
-                 help="Load local domain folder and load into reference track, download from "
+                 help="Load local domain folder and load into annotation track, download from "
                       "https://hgdownload.soe.ucsc.edu/gbdb/hg38/uniprot/")
 @optgroup.option("--domain-include", default=None, type=click.STRING, show_default=True,
-                 help="Which domain will be included in reference plot")
+                 help="Which domain will be included in annotation plot")
 @optgroup.option("--domain-exclude", default=None, type=click.STRING, show_default=True,
-                 help="Which domain will be excluded in reference plot")
+                 help="Which domain will be excluded in annotation plot")
 @optgroup.option("--remove-empty", is_flag=True, type=click.BOOL, show_default=True,
                  help="Whether to plot empty transcript")
 @optgroup.option("--transcripts-to-show", default="", show_default=True,
@@ -472,58 +435,73 @@ def process_file_list(infile: str, category: str = "density"):
 @optgroup.group("Layout settings")
 @optgroup.option("--n-y-ticks", default=4, type=click.IntRange(min=0, clamp=True),
                  help="The number of ticks of y-axis")
-@optgroup.option("--distance-ratio", type=click.FLOAT, default=0.1,
+@optgroup.option("--distance-ratio", type=click.FLOAT, default=0,
                  help="The distance between transcript label and transcript line", show_default=True)
-@optgroup.option("--reference-scale", type=click.FLOAT, default=.25,
-                 help="The size of reference plot in final plot", show_default=True)
+@optgroup.option("--annotation-scale", type=click.FLOAT, default=.25,
+                 help="The size of annotation plot in final plot", show_default=True)
 @optgroup.option("--stroke-scale", type=click.FLOAT, default=.25,
                  help="The size of stroke plot in final image", show_default=True)
 @optgroup.group("Overall settings")
 @optgroup.option("--font-size", default=8, type=click.IntRange(min=1, clamp=True),
                  help="The font size of x, y-axis and so on")
 @optgroup.option("--reverse-minus", default=False, is_flag=True, type=click.BOOL,
-                 help="Whether to reverse strand of bam/reference file")
+                 help="Whether to reverse strand of bam/annotation file")
 @optgroup.option("--hide-y-label", default=False, is_flag=True, type=click.BOOL,
                  help="Whether hide y-axis label")
 @optgroup.option("--same-y", default=False, is_flag=True, type=click.BOOL,
                  help="Whether different density/line plots shared same y-axis boundaries")
 @optgroup.option("--same-y-sc", default=False, is_flag=True, type=click.BOOL,
                  help="Similar with --same-y, but only shared same y-axis boundaries between same single cell files")
+@optgroup.option("--same-y-by-groups", type=click.Path(), default=None,
+                 help="""
+                 Set groups for --same-y, this parameter is path to a file with 2 columns, 
+                 - 1st is the label to specific input,
+                 - 2nd the the group labels
+                 - the input files not listed will use the global y limits
+                 """
+                 )
+@optgroup.option("--y-limit", default=None, type=click.Path(), help="""
+                 Manully set the y limit for all plots supported, this parameter is path to a file with 3 columns,
+                 - 1st is the label to specific input,
+                 - 2nd the maximum y
+                 - 3rd the minimum y
+                 - the input files not listed will use the default
+                 """)
 @optgroup.option('--log', type=click.Choice(["0", "2", "10", "zscore"]), default="0",
                  help="y axis log transformed, 0 -> not log transform;2 -> log2;10 -> log10")
 @optgroup.option("--title", type=click.STRING, default=None, help="Title", show_default=True)
 @optgroup.option("--font", type=click.STRING, default=None, help="Fonts", show_default=True)
+@optgroup.group("Web settings")
+@optgroup.option("--start-server", type=click.BOOL, is_flag=True, help="Start web ui instead of running in cmd mode")
+@optgroup.option("--host", type=click.STRING, default="localhost", help="The ip address binding to")
+@optgroup.option("--port", type=click.INT, default=5000, help="The port to listen on")
+@optgroup.option("--plots", type=click.Path(), default=__PLOT__,
+              help="The path to directory where to save the backend plot data and logs, required while using appImage.")
+@optgroup.option("--data", type=click.Path(exists=True), default="./",
+              help="The path to directory contains all necessary data files.")
 def main(**kwargs):
     u"""
     Welcome to use trackplot
     \f
     """
     logger.remove()
-    logger.add(sys.stderr, level="INFO" if not kwargs["debug"] else "DEBUG")
+    logger.add(sys.stderr, level="DEBUG" if kwargs["verbose"] else "INFO")
+    logger.debug("DEBUG" if kwargs["verbose"] else "INFO")
+
+    if kwargs["start_server"]:
+        run(host=kwargs["host"], port=kwargs["port"], plots=kwargs["plots"], data=kwargs["data"])
+        exit(0)
+    else:
+        assert kwargs["event"], "-e, --event required"
 
     # print warning info about backend
     if (kwargs["domain"] or kwargs["local_domain"]) and kwargs["backend"].lower() != "cairo":
-        logger.warning(f"{kwargs['backend']} backend may have problems with small domain, "
-                       f"if there is any please try cairo backend instead.")
+        logger.debug(f"{kwargs['backend']} backend may have problems with small domain, "
+                     f"if there is any please try cairo backend instead.")
 
     if kwargs["raster"] and kwargs["heatmap"] and kwargs["backend"].lower() == "cairo":
-        logger.warning(f"{kwargs['backend']} backend may have problems with rasterized heatmap, "
-                       f"if there is any, please try another backend instead.")
-
-    try:
-        mpl.use(kwargs["backend"])
-    except ImportError as err:
-        if kwargs["backend"].lower() == "cairo":
-            logger.warning("Cairo backend required cairocffi installed")
-            logger.warning("Switch back to Agg backend")
-        else:
-            logger.warning(f"backend error, switch back to Agg: {err}")
-        mpl.use("Agg")
-
-    mpl.rcParams['pdf.fonttype'] = 42
-
-    if kwargs["font"]:
-        mpl.rcParams['font.family'] = kwargs["font"]
+        logger.debug(f"{kwargs['backend']} backend may have problems with rasterized heatmap, "
+                     f"if there is any, please try another backend instead.")
 
     for k, v in kwargs.items():
         logger.debug(f"{k} => {v}")
@@ -533,7 +511,7 @@ def main(**kwargs):
     else:
         included_junctions = {}
 
-    p = Plot()
+    p = Plot(logfile=kwargs["logfile"], font_family=kwargs["font"], backend=kwargs["backend"])
 
     region = decode_region(kwargs["event"])
     p.set_region(region=region)
@@ -545,25 +523,25 @@ def main(**kwargs):
 
     size_factors = {}
 
-    # add reference
+    # add annotation
+    # print(kwargs.keys())
     for key in kwargs.keys():
         if key in IMAGE_TYPE and kwargs[key] and os.path.exists(kwargs[key]):
-            logger.debug(f"add {key} {kwargs[key]}")
-            if key == "reference":
-                p.set_reference(kwargs["reference"],
-                                show_gene=not kwargs["no_gene"],
-                                color=kwargs["ref_color"],
-                                remove_empty_transcripts=kwargs["remove_empty"],
-                                choose_primary=kwargs["choose_primary"],
-                                font_size=kwargs["font_size"],
-                                show_id=kwargs["show_id"],
-                                show_exon_id=kwargs["show_exon_id"],
-                                transcripts=kwargs["transcripts_to_show"],
-                                add_domain=kwargs["domain"],
-                                local_domain=kwargs["local_domain"],
-                                domain_include=kwargs["domain_include"],
-                                domain_exclude=kwargs["domain_exclude"]
-                                )
+            if key == "annotation":
+                p.set_annotation(kwargs["annotation"],
+                                 show_gene=not kwargs["no_gene"],
+                                 color=kwargs["ref_color"],
+                                 remove_empty_transcripts=kwargs["remove_empty"],
+                                 choose_primary=kwargs["choose_primary"],
+                                 font_size=kwargs["font_size"],
+                                 show_id=kwargs["show_id"],
+                                 show_exon_id=kwargs["show_exon_id"],
+                                 transcripts=kwargs["transcripts_to_show"],
+                                 add_domain=kwargs["domain"],
+                                 local_domain=kwargs["local_domain"],
+                                 domain_include=kwargs["domain_include"],
+                                 domain_exclude=kwargs["domain_exclude"]
+                                 )
             elif key == "interval":
                 for f in process_file_list(kwargs[key], key):
                     p.add_interval(f.path, f.label)
@@ -738,7 +716,6 @@ def main(**kwargs):
                               )
             elif key == "hic":
                 for f in process_file_list(kwargs[key], "hic"):
-
                     p.add_hic(
                         f.path,
                         category=f.category,
@@ -783,10 +760,12 @@ def main(**kwargs):
         raster=kwargs["raster"],
         intron_scale=kwargs["intron_scale"],
         exon_scale=kwargs["exon_scale"],
-        reference_scale=kwargs["reference_scale"],
+        annotation_scale=kwargs["annotation_scale"],
         stroke_scale=kwargs["stroke_scale"],
         same_y=kwargs["same_y"],
         same_y_sc=kwargs.get("same_y_sc", False),
+        same_y_groups=kwargs.get("same_y_by_groups", None),
+        y_limit=kwargs.get("y_limit", None),
         remove_duplicate_umi=kwargs["remove_duplicate_umi"],
         threshold=kwargs["threshold"],
         sc_height_ratio={
@@ -800,7 +779,6 @@ def main(**kwargs):
         fill_step=kwargs.get("fill_step", "post"),
         smooth_bin=kwargs["smooth_bin"]
     )
-    logger.info("DONE")
 
 
 if __name__ == '__main__':
