@@ -273,6 +273,8 @@ class ReadSegment(File):
         get a nested list which presents the order of plot
         :return:
         """
+        import warnings
+        warnings.simplefilter(action='ignore', category=FutureWarning)
         assert self.meta is not None, f"Not found meta information, please `load` first."
         for ind in self.meta.groupby(['y_loci'])['list_index'].apply(list).tolist()[::-1]:
             yield ind
@@ -516,7 +518,7 @@ class ReadSegment(File):
                 mtx[i, (e_start - self.region.start + 1):(e_end - self.region.start + 1)] = 1
 
         # "single", "complete", "average", "weighted", "centroid", "median", "ward"
-        order = dendrogram(linkage(mtx, method="centroid", metric="euclidean"))
+        order = dendrogram(linkage(mtx, method="centroid", metric="euclidean"), no_plot=True)
 
         data = []
         for i in order["leaves"]:
