@@ -8,14 +8,14 @@ import faulthandler
 import io
 import logging
 import os
+from datetime import datetime
 from multiprocessing import Pool
-from typing import Set
 
-import matplotlib.pyplot as plt
 from matplotlib import gridspec
 from matplotlib.backends.backend_agg import FigureCanvasAgg
 from matplotlib.backends.backend_pdf import FigureCanvasPdf
 
+from trackplot.file.ATAC import ATAC
 from trackplot.file.Bam import Bam
 from trackplot.file.BedGraph import Bedgraph
 from trackplot.file.Bigwig import Bigwig
@@ -30,6 +30,11 @@ logging.getLogger('matplotlib.font_manager').setLevel(logging.ERROR)
 faulthandler.enable()
 
 
+__version__ = "0.5.0"
+__author__ = "ygidtu & Ran Zhou"
+__email__ = "ygidtu@gmail.com"
+
+
 def __load__(args):
     p, args, kwargs = args
     p.load(*args, **kwargs)
@@ -96,17 +101,28 @@ class PlotInfo(object):
                 data[obj.label] = obj.data
         return data
 
-    def len(self, scale: Union[int, float] = .25) -> int:
+    def len(self, scale: Union[int, float] = .25, sc_height_ratio: Optional[Dict[str, float]] = None) -> Tuple[int, List]:
         n = 0
+        height_ratio = [1]
+
+        if self.is_single_cell:
+            if sc_height_ratio is None:
+                sc_height_ratio = {"density": .2, "heatmap": .2}
+            height_ratio = [sc_height_ratio.get(self.type, 1)]
+
         if not self.category:
             pass
         elif self.type == "site-plot" and self.category[0] == "bam":
-            n += 2
+            n += 2 if not isinstance(self.obj[0], Depth) else 2 * len(self.obj[0])
         elif self.type == "igv":
+            # seems igv scale will produce float
             n += self.obj[0].len(scale / 8)
+        elif isinstance(self.obj[0], Depth):
+            n += len(self.obj[0])
         else:
             n += 1
-        return n
+
+        return int(n), [height_ratio[0] for _ in range(n)]
 
     def add(self, obj: File, category: str = "", type_: str = ""):
         u"""
@@ -128,6 +144,9 @@ class PlotInfo(object):
                 kwargs_["region"] = region
                 self.obj = p.map(__load__, [[o, args, kwargs_] for o in self.obj])
 
+        for obj in self.obj:
+            obj.transform()
+
         if self.type == "density":
             if len(self.obj) > 1:
                 data = self.obj[0].data
@@ -140,6 +159,18 @@ class PlotInfo(object):
         return self
 
 
+def add_object_error(func):
+    def inner(*args, **kwargs):
+        # calling the actual function now
+        # inside the wrapper function.
+        try:
+            func(*args, **kwargs)
+        except Exception as err:
+            logger.debug(f"trackplot will ignore this object, {err}")
+
+    return inner
+
+
 class Plot(object):
     u"""
     this class is the main framework of sashimi
@@ -148,11 +179,14 @@ class Plot(object):
     __slots__ = [
         "__n_objs__", "region", "sites",
         "focus", "stroke", "events",
-        "sequence", "reference", "graph_coords",
+        "sequence", "annotation", "graph_coords",
         "plots", "params", "junctions", "link"
     ]
 
-    def __init__(self):
+    def __init__(self,
+                 logfile: Optional[str] = None,
+                 backend: Optional[str] = None,
+                 font_family: Optional[str] = None):
         u"""
         init this class
         """
@@ -163,13 +197,34 @@ class Plot(object):
         self.stroke = []
         self.events = None
         self.sequence = None
-        self.reference = None
+        self.annotation = None
         self.graph_coords = None
         self.plots = []
         self.params = {}
         self.junctions = {}
         self.link = []
 
+        if logfile:
+            logger.add(logfile, level="DEBUG")
+
+        logger.info(f"Create trackplot version: {__version__}")
+
+        # print warning info about backend
+        if backend:
+            try:
+                plt.switch_backend(backend)
+            except ImportError as err:
+                if backend.lower() == "cairo":
+                    logger.debug("Cairo backend required cairocffi installed")
+                    logger.debug("Switch back to Agg backend")
+                else:
+                    logger.debug(f"backend error, switch back to Agg: {err}")
+                plt.switch_backend("Agg")
+
+        plt.rcParams['pdf.fonttype'] = 42
+        if font_family:
+            plt.rcParams['font.family'] = font_family
+
     @property
     def chrom(self) -> Optional[str]:
         if self.region:
@@ -192,9 +247,10 @@ class Plot(object):
 
     @property
     def exons(self) -> Optional[List[List[int]]]:
-        if self.reference:
-            return self.reference.exons
+        if self.annotation:
+            return self.annotation.exons
 
+    @add_object_error
     def set_region(self, chromosome: str = "",
                    start: int = 0, end: int = 0,
                    strand: str = "+",
@@ -210,6 +266,7 @@ class Plot(object):
         logger.info(f"set region to {self.region}")
         return self
 
+    @add_object_error
     def add_sites(self, sites):
         u"""
         highlight specific sites
@@ -217,7 +274,9 @@ class Plot(object):
         :return:
         """
         assert self.region is not None, f"please set plot region first."
+
         if sites is not None:
+            logger.info(f"add sites: {sites}")
             if isinstance(sites, str):
                 sites = [int(x) for x in sites.split(",")]
 
@@ -235,7 +294,8 @@ class Plot(object):
                     self.sites[sites] = "red"
         return self
 
-    def add_focus(self, focus: Optional[str], start: int = 0, end: int = 0):
+    @add_object_error
+    def add_focus(self, focus: Optional[str] = None, start: int = 0, end: int = 0):
         u"""
         set focus region
         :param focus: string in 100-200:300-400
@@ -245,6 +305,7 @@ class Plot(object):
         """
         assert self.region is not None, f"please set plot region first."
         if focus:
+            logger.info(f"add focus: {focus}")
             for site in focus.split(":"):
                 site = sorted([int(x) - self.start for x in site.split("-")])
                 if site[0] < 0:
@@ -256,9 +317,11 @@ class Plot(object):
                 self.focus[site[0]] = max(site[1], self.focus.get(site[0], -1))
 
         if 0 < start < end:
+            logger.info(f"add focus: {start}-{end}")
             self.focus[start] = max(end, self.focus.get(start, -1))
         return self
 
+    @add_object_error
     def add_stroke(
             self,
             stroke: Optional[str] = None,
@@ -278,12 +341,15 @@ class Plot(object):
         """
         assert self.region is not None, f"please set plot region first."
         if stroke:
+            logger.info(f"add stroke: {stroke}")
             self.stroke += Stroke.create(stroke, self.region)
 
         if 0 < start < end:
+            logger.info(f"add stroke: {start}-{end}")
             self.stroke.append(Stroke(start - self.start, end - self.end, color, label))
         return self
 
+    @add_object_error
     def add_links(
             self,
             link: Optional[str] = None,
@@ -302,13 +368,17 @@ class Plot(object):
         :return:
         """
         assert self.region is not None, f"please set plot region first."
+
         if link:
+            logger.info(f"add link: {link}")
             self.link.append(Stroke.create(link, self.region, default_color=color))
 
         if 0 < start < end:
+            logger.info(f"add link: {start}-{end}")
             self.link.append([Stroke(start - self.start, end - self.end, color, label)])
         return self
 
+    @add_object_error
     def set_sequence(self, fasta: str):
         u"""
         set sequence info for
@@ -319,33 +389,35 @@ class Plot(object):
         self.sequence = Fasta.create(fasta)
         return self
 
-    def set_reference(self, gtf: str,
-                      add_domain: bool = False,
-                      local_domain: Optional[str] = False,
-                      domain_include: Optional[str] = False,
-                      domain_exclude: Optional[str] = False,
-                      interval: Optional[str] = None,
-                      interval_label: Optional[str] = None,
-                      transcripts: Optional[List[str]] = None,
-                      remove_empty_transcripts: bool = False,
-                      choose_primary: bool = False,
-                      color: Optional[str] = "black",
+    @add_object_error
+    def set_annotation(self, gtf: str,
+                       add_domain: bool = False,
+                       local_domain: Optional[str] = False,
+                       domain_include: Optional[str] = False,
+                       domain_exclude: Optional[str] = False,
+                       interval: Optional[str] = None,
+                       interval_label: Optional[str] = None,
+                       transcripts: Optional[List[str]] = None,
+                       remove_empty_transcripts: bool = False,
+                       choose_primary: bool = False,
+                       color: Optional[str] = "black",
 
-                      # transcripts related parameters
-                      font_size: int = 5,
-                      show_gene: bool = False,
-                      show_id: bool = False,
-                      exon_width: float = .3,
-                      show_exon_id: bool = False,
-                      theme: str = "blank"
-                      ):
+                       # transcripts related parameters
+                       font_size: int = 5,
+                       show_gene: bool = True,
+                       show_id: bool = False,
+                       exon_width: float = .3,
+                       show_exon_id: bool = False,
+                       theme: str = "blank",
+                       **kwargs
+                       ):
         u"""
         add transcripts to this region
         :param gtf: the path of gtf file
-        :param add_domain: whether add domain information into reference plot
-        :param local_domain: add a local domain file into reference plot
-        :param domain_exclude: the domain will be included in reference plot
-        :param domain_include: the domain will be excluded in reference plot
+        :param add_domain: whether add domain information into annotation plot
+        :param local_domain: add a local domain file into annotation plot
+        :param domain_exclude: the domain will be included in annotation plot
+        :param domain_include: the domain will be excluded in annotation plot
         :param interval: the path of interval annotation file, such as polyAsites
         :param interval_label: the label of current interval annotation file
         :param font_size: the size of transcript id, name
@@ -360,8 +432,8 @@ class Plot(object):
         :param show_exon_id: whether to show exon id
         :return:
         """
-        logger.info(f"set reference file to {gtf}")
-        self.reference = Reference.create(
+        logger.info(f"set annotation file to {gtf}")
+        self.annotation = Annotation.create(
             gtf,
             add_domain=add_domain,
             add_local_domain=local_domain,
@@ -370,9 +442,9 @@ class Plot(object):
         )
 
         if interval and interval_label:
-            self.reference.add_interval(interval, interval_label)
+            self.add_interval(interval, interval_label)
 
-        self.params["reference"] = {
+        self.params["annotation"] = {
             "transcripts": transcripts,
             "remove_empty_transcripts": remove_empty_transcripts,
             "choose_primary": choose_primary,
@@ -387,9 +459,11 @@ class Plot(object):
 
         return self
 
+    @add_object_error
     def add_interval(self, interval: str, interval_label: str):
-        assert self.reference is not None, "please set_reference first."
-        self.reference.add_interval(interval, interval_label)
+        assert self.annotation is not None, "please set_annotation first."
+        logger.info(f"add interval: {interval} - {interval_label}")
+        self.annotation.add_interval(interval, interval_label)
         return self
 
     def __init_input_file__(self, path: str,
@@ -415,9 +489,10 @@ class Plot(object):
                             size_factor=None,
 
                             log_trans: Optional[str] = None,
-                            ):
+                            ) -> Tuple[File, str]:
         self.__n_objs__ += 1
         path = os.path.expanduser(path)
+        logger.info(f"add {category} {label} {path}")
         if category == "bam":
             obj = Bam.create(
                 path,
@@ -449,7 +524,7 @@ class Plot(object):
                 del_ratio_ignore=del_ratio_ignore,
                 exon_focus=exon_focus
             )
-        elif category == "hic":
+        elif category == "hic" or category == "h5":
             obj = HiCTrack.create(
                 path=path,
                 label=label,
@@ -467,15 +542,17 @@ class Plot(object):
             obj = Depth.create(path, label=label, title=title)
         else:
             raise ValueError(
-                f"the category should be one of [bam, bigwig, bw, depth, bedgraph, bg], instead of {category}")
+                f"the category should be one of [bam, bigwig, bw, depth, bedgraph, bg, h5], instead of {category}")
 
         obj.log_trans = log_trans
         return obj, category
 
+    @add_object_error
     def add_customized_junctions(self, path: str):
         if path and os.path.exists(path) and os.path.isfile(path):
             self.junctions = load_custom_junction(path)
 
+    @add_object_error
     def add_density(self,
                     path: str,
                     category: str = "bam",
@@ -553,7 +630,8 @@ class Plot(object):
         if show_site_plot and category == "bam":
             type_ = "site-plot"
         elif show_site_plot:
-            logger.warning("show_site_plot only works with bam files")
+            if category != "bam":
+                raise ValueError("show_site_plot only works with bam files")
 
         info = PlotInfo(obj=obj, type_=type_, category=category)
 
@@ -587,6 +665,7 @@ class Plot(object):
             }
         return self
 
+    @add_object_error
     def add_heatmap(self,
                     path: str,
                     group: str = "",
@@ -686,6 +765,7 @@ class Plot(object):
 
         return self
 
+    @add_object_error
     def add_line(self,
                  path: str,
                  group: str = "",
@@ -778,6 +858,7 @@ class Plot(object):
 
         return self
 
+    @add_object_error
     def add_hic(
             self,
             path: str,
@@ -813,9 +894,9 @@ class Plot(object):
             "show_y_label": show_y_label,
             "theme": theme
         }
-
         return self
 
+    @add_object_error
     def add_igv(
             self,
             path: str,
@@ -861,7 +942,7 @@ class Plot(object):
         """
         obj, category = self.__init_input_file__(
             path=path,
-            category=category,
+            category="igv",
             library=library,
             features=features,
             deletion_ignore=deletion_ignore,
@@ -885,6 +966,7 @@ class Plot(object):
 
         return self
 
+    @add_object_error
     def add_motif(self,
                   path: str,
                   category: str = "motif",
@@ -907,6 +989,7 @@ class Plot(object):
         self.params[info] = {"width": width, "theme": theme}
         return self
 
+    @add_object_error
     def add_manual(self,
                    data: np.array,
                    image_type: str = "line",
@@ -974,7 +1057,7 @@ class Plot(object):
 
     def plot(self,
              output: Optional[str] = None,
-             reference_scale: Union[int, float] = .25,
+             annotation_scale: Union[int, float] = .25,
              stroke_scale: Union[int, float] = .25,
              dpi: int = 300,
              width: Union[int, float] = 0,
@@ -988,7 +1071,7 @@ class Plot(object):
         u"""
         draw image
         :param output: if output is empty then show this image by plt.showfig
-        :param reference_scale: to adjust the size of reference plot
+        :param annotation_scale: to adjust the size of annotation plot
         :param stroke_scale: to adjust the size of stroke plot
         :param dpi: the dpi of saved plot
         :param width: the width of figure, if width == 0, the let matplotlib decide the size of image
@@ -1004,14 +1087,13 @@ class Plot(object):
             sc_height_ratio = {"density": .2, "heatmap": .2}
         assert self.region is not None, f"please set the plotting region first."
 
-        plots_n_rows = 1
-        plots_n_cols = 1
+        plots_n_rows, plots_n_cols = 1, 1
 
         height_ratio = []
-        if self.reference is not None:
-            logger.info("load reference")
-            self.reference.load(self.region, *args, **kwargs)
-            plots_n_rows += self.reference.len(scale=reference_scale)
+        if self.annotation is not None:
+            logger.info("load annotation")
+            self.annotation.load(self.region, *args, **self.params["annotation"])
+            plots_n_rows += self.annotation.len(scale=annotation_scale)
 
         if self.stroke:
             plots_n_rows += int(max(len(self.stroke) * stroke_scale, 1))
@@ -1030,11 +1112,24 @@ class Plot(object):
             assert isinstance(p, PlotInfo), f"unrecognized data type: {type(p)}"
             # only enable the multiprocessing while n_jobs > 1
             if self.__n_objs__ / len(self.plots) >= n_jobs > 1:
-                p.load(self.region, n_jobs, junctions=self.junctions.get(p.obj[0].label, {}), *args, **kwargs)
+                if isinstance(p.obj[0].label, list):
+                    juncs = {}
+                    for i in p.obj[0].label:
+                        juncs.update(self.junctions.get(i, {}))
+                    p.load(self.region, junctions=juncs, *args, **kwargs)
+                else:
+                    p.load(self.region, n_jobs, junctions=self.junctions.get(p.obj[0].label, {}), *args, **kwargs)
             elif n_jobs > 1:
                 temp = copy.deepcopy(kwargs)
                 temp["region"] = self.region if p.type != "motif" else self.params[p]["region"]
-                temp["junctions"] = self.junctions.get(p.obj[0].label, {})
+
+                if isinstance(p.obj[0].label, list):
+                    juncs = {}
+                    for i in p.obj[0].label:
+                        juncs.update(self.junctions.get(i, {}))
+                    temp["junctions"] = juncs
+                else:
+                    temp["junctions"] = self.junctions.get(p.obj[0].label, {})
                 cmds.append([p, args, temp])
 
         if len(cmds) > 0:
@@ -1044,41 +1139,33 @@ class Plot(object):
         # count the plots size
         for p in self.plots:
             if n_jobs <= 1:
-                p.load(self.region, junctions=self.junctions.get(p.obj[0].label, {}), *args, **kwargs)
+                if isinstance(p.obj[0].label, list):
+                    juncs = {}
+                    for i in p.obj[0].label:
+                        juncs.update(self.junctions.get(i, {}))
+                    p.load(self.region, junctions=juncs, *args, **kwargs)
+                else:
+                    p.load(self.region, junctions=self.junctions.get(p.obj[0].label, {}), *args, **kwargs)
 
-            for obj in p.obj:
-                if isinstance(obj, ReadSegment):
-                    continue
-                if isinstance(obj, HiCTrack):
-                    continue
-                obj.transform()
+            n_rows, n_height = p.len(annotation_scale, sc_height_ratio=sc_height_ratio)
+            plots_n_rows += n_rows
+            height_ratio += n_height
 
-            plots_n_rows += p.len(reference_scale)
             if p.type in ["heatmap", "hic"]:
                 plots_n_cols = 2
 
-            if p.is_single_cell:
-                height_ratio.append(sc_height_ratio.get(p.type, 1))
-            elif p.type == "igv":
-                height_ratio.append(p.len(reference_scale))
-            else:
-                height_ratio.append(1)
+        plots_n_rows = int(plots_n_rows)
+        height_ratio += [1 for _ in range(plots_n_rows - len(height_ratio))]
 
         logger.debug(f"plots n_rows={plots_n_rows}; n_cols = {plots_n_cols}")
         logger.info("init graph_coords")
         exon_scale = kwargs.get("exon_scale", 1)
         intron_scale = kwargs.get("intron_scale", .5)
         if plots_n_cols > 1 and intron_scale != 1:
-            logger.warning(f"heatmap require intron_scale = 1")
+            logger.debug(f"heatmap require intron_scale = 1")
             intron_scale = 1
 
-        self.graph_coords = init_graph_coords(
-            self.region, self.exons,
-            exon_scale=exon_scale,
-            intron_scale=intron_scale
-        )
-
-        height_ratio += [1 for _ in range(plots_n_rows - len(height_ratio))]
+        self.graph_coords = init_graph_coords(self.region, self.exons, exon_scale=exon_scale, intron_scale=intron_scale)
 
         if width and height:
             fig = plt.figure(figsize=[width, height * sum(height_ratio)], dpi=dpi)
@@ -1092,43 +1179,131 @@ class Plot(object):
             gs = gridspec.GridSpec(plots_n_rows, plots_n_cols, height_ratios=height_ratio,
                                    wspace=.7, hspace=.15)
 
-        max_used_y_val, min_used_y_val = {}, {}
+        max_used_y_val, min_used_y_val, same_y_by_groups = {}, {}, {}
+        
+        default_y = {}
+        if kwargs.get("y_limit") and os.path.exists(kwargs.get("y_limit")):
+            default_y = {}
+            with open(kwargs.get("y_limit")) as r:
+                for line in r:
+                    if line.startswith("#"):
+                        continue
+                    line = line.split()
+                    
+                    if len(line) > 1:
+                        try:
+                            default_y[line[0]] = [float(x) for x in default_y[1:]]
+                        except Exception as err:
+                            logger.warning(f"The y limit of {line[0]} is invalid: {err}")
+                        
+            
         if kwargs.get("same_y") or kwargs.get("same_y_sc"):
+            logger.info("--same-y is enabled")
+            # read the y groups            
+            if kwargs.get("same_y_groups") and os.path.exists(kwargs.get("same_y_groups")):
+                logger.info("Reading the same y by groups from: %s" % kwargs.get("same_y_groups"))
+                with open(kwargs.get("same_y_groups")) as r:
+                    for line in r:
+                        if line.startswith("#"):
+                            continue
+                        
+                        line = line.split()
+
+                        # use hashlib to avoid potential repeat group names
+                        same_y_by_groups[line[0]] = f"samy_y_groups_of_{line[1]} {datetime.now()}"
+            
             for p in self.plots:
                 if p.type in ["density", "site-plot", "line"]:
                     for obj in p.obj:
-                        max_used_y_val[obj.path] = max(max(obj.data.wiggle), max_used_y_val.get(obj.path, 0))
+                        if kwargs.get("y_max") is not None:
+                            logger.info("")
 
+                        # if object is assigned y groups then record one group y axis
+                        if obj.label in same_y_by_groups:
+                            key = same_y_by_groups[obj.label]
+                            
+                            max_used_y_val[key] = max(max(obj.data.wiggle), max_used_y_val.get(key, 0))
+                            if obj.data.minus is None:
+                                min_used_y_val[key] = min(min(obj.data.wiggle), min_used_y_val.get(key, 0))
+                            else:
+                                min_used_y_val[key] = min(min(obj.data.minus), min_used_y_val.get(obj.path, 0))
+                            
+                        max_used_y_val[obj.path] = max(max(obj.data.wiggle), max_used_y_val.get(obj.path, 0))
                         if obj.data.minus is None:
                             min_used_y_val[obj.path] = min(min(obj.data.wiggle), min_used_y_val.get(obj.path, 0))
                         else:
                             min_used_y_val[obj.path] = min(min(obj.data.minus), min_used_y_val.get(obj.path, 0))
 
+                        # set y limit
+                        if obj.label in default_y:
+                            max_used_y_val[obj.path] = default_y[obj.label][0]
+                            
+                            if len(default_y[obj.label]) > 1:
+                                min_used_y_val[obj.path] = default_y[obj.label][1]
+                            
+                            if same_y_by_groups and obj.label in same_y_by_groups:
+                                key = same_y_by_groups[obj.label]
+                                max_used_y_val[key] = max_used_y_val[obj.path]
+                                min_used_y_val[key] = min_used_y_val[obj.path]
+
         curr_idx = 0
         for p in self.plots:
             if p.type == "igv":
-                ax_var = plt.subplot(gs[curr_idx: curr_idx + p.len(reference_scale), 0])
+                ax_var = plt.subplot(gs[curr_idx: curr_idx + p.len(annotation_scale)[0], 0])
             else:
                 ax_var = plt.subplot(gs[curr_idx, 0])
 
+            if curr_idx == 0:
+                ax_var.set_title(str(self.region), loc="left")
+
             max_y_val_, min_y_val_ = None, None
+            
             if kwargs.get("same_y_sc") and p.obj[0].is_single_cell:
                 max_y_val_, min_y_val_ = max_used_y_val[p.obj[0].path], min_used_y_val[p.obj[0].path]
+                logger.info(f"Set for {p.obj[0].label} by single cell: max_y={max_y_val_}; min_y={min_y_val_}")
+            elif kwargs.get("same_y_groups") and p.obj[0].label in same_y_by_groups:
+                max_y_val_, min_y_val_ = max_used_y_val[same_y_by_groups[p.obj[0].label]], min_used_y_val[same_y_by_groups[p.obj[0].label]]
+                logger.info(f"Set for {p.obj[0].label} by group {same_y_by_groups[p.obj[0].label]}: max_y={max_y_val_}; min_y={min_y_val_}")
             elif kwargs.get("same_y"):
                 max_y_val_, min_y_val_ = max(max_used_y_val.values()), min(min_used_y_val.values())
+                logger.info(f"Set for {p.obj[0].label} by global: max_y={max_y_val_}; min_y={min_y_val_}")
 
+            logger.info(f"plotting {p.type} at idx: {curr_idx} with height_ratio: {height_ratio[curr_idx]}")
             if p.type == "density":
-                plot_density(
-                    ax=ax_var,
-                    obj=p.obj[0],
-                    graph_coords=self.graph_coords,
-                    max_used_y_val=max_y_val_,
-                    min_used_y_val=min_y_val_,
-                    distance_between_label_axis=distance_between_label_axis,
-                    raster=raster,
-                    fill_step=fill_step,
-                    **self.params[p]
-                )
+                if isinstance(p.obj[0], Depth):
+                    for key, readDepth in p.obj[0].data.items():
+                        temp_params = self.params.get(p, {})
+
+                        if "y_label" not in temp_params:
+                            temp_params["y_label"] = key
+
+                        plot_density(
+                            ax=ax_var,
+                            data=readDepth,
+                            region=self.region,
+                            graph_coords=self.graph_coords,
+                            max_used_y_val=max_y_val_,
+                            min_used_y_val=min_y_val_,
+                            distance_between_label_axis=distance_between_label_axis,
+                            raster=raster,
+                            fill_step=fill_step,
+                            **temp_params
+                        )
+                        curr_idx += 1
+                        ax_var = plt.subplot(gs[curr_idx, 0])
+                    curr_idx -= 1
+                else:
+                    plot_density(
+                        ax=ax_var,
+                        obj=p.obj[0],
+                        graph_coords=self.graph_coords,
+                        max_used_y_val=max_y_val_,
+                        min_used_y_val=min_y_val_,
+                        distance_between_label_axis=distance_between_label_axis,
+                        raster=raster,
+                        fill_step=fill_step,
+                        **self.params.get(p, {})
+                    )
             elif p.type == "hic":
                 plot_hic(
                     ax=ax_var,
@@ -1136,7 +1311,7 @@ class Plot(object):
                     obj=p.obj,
                     distance_between_label_axis=distance_between_label_axis,
                     raster=raster,
-                    **self.params[p]
+                    **self.params.get(p, {})
                 )
             elif p.type == "site-plot":
                 plot_density(
@@ -1147,19 +1322,17 @@ class Plot(object):
                     min_used_y_val=min_y_val_,
                     distance_between_label_axis=distance_between_label_axis,
                     raster=raster,
-                    **self.params[p]
+                    **self.params.get(p, {})
                 )
 
-                site_ax = plt.subplot(gs[curr_idx + 1, 0])
-
+                curr_idx += 1
                 plot_site_plot(
-                    site_ax, p.obj[0],
+                    plt.subplot(gs[curr_idx, 0]), p.obj[0],
                     graph_coords=self.graph_coords,
                     raster=raster,
                     distance_between_label_axis=distance_between_label_axis,
-                    **self.params[p]
+                    **self.params.get(p, {})
                 )
-                curr_idx += 1
             elif p.type == "heatmap":
                 plot_heatmap(
                     ax=ax_var,
@@ -1169,7 +1342,7 @@ class Plot(object):
                     graph_coords=self.graph_coords,
                     raster=raster,
                     distance_between_label_axis=distance_between_label_axis,
-                    **self.params[p]
+                    **self.params.get(p, {})
                 )
             elif p.type == "line":
                 plot_line(
@@ -1180,7 +1353,7 @@ class Plot(object):
                     min_used_y_val=min_y_val_,
                     graph_coords=self.graph_coords,
                     distance_between_label_axis=distance_between_label_axis,
-                    **self.params[p]
+                    **self.params.get(p, {})
                 )
             elif p.type == "igv":
                 plot_igv_like(
@@ -1189,7 +1362,7 @@ class Plot(object):
                     graph_coords=self.graph_coords,
                     raster=raster,
                     distance_between_label_axis=distance_between_label_axis,
-                    **self.params[p]
+                    **self.params.get(p, {})
                 )
             elif p.type == "motif":
                 plot_motif(ax=ax_var, obj=p.obj[0], graph_coords=self.graph_coords, **self.params[p])
@@ -1203,15 +1376,17 @@ class Plot(object):
             if p.type != "igv":
                 curr_idx += 1
             else:
-                curr_idx += p.len(reference_scale)
+                curr_idx += p.len(annotation_scale)[0]
 
         if self.link:
+            logger.info(f"plotting links at idx: {curr_idx} with height_ratio: {height_ratio[curr_idx]}")
             for link in self.link:
                 plot_links(ax=plt.subplot(gs[curr_idx:(curr_idx + 1), 0]),
                            data=link, graph_coords=self.graph_coords)
                 curr_idx += 1
 
         # draw x label
+        logger.info(f"plotting x-axis ticks at idx: {curr_idx} with height_ratio: {height_ratio[curr_idx]}")
         set_x_ticks(
             ax=plt.subplot(gs[curr_idx, 0]), region=self.region,
             graph_coords=self.graph_coords,
@@ -1220,24 +1395,28 @@ class Plot(object):
         )
         curr_idx += 1
 
-        if self.reference is not None:
-            ax_var = plt.subplot(gs[curr_idx:curr_idx + self.reference.len(scale=reference_scale), 0])
-            plot_reference(ax=ax_var, obj=self.reference,
-                           graph_coords=self.graph_coords,
-                           plot_domain=self.reference.add_domain,
-                           distance_between_label_axis=distance_between_label_axis,
-                           **self.params["reference"])
+        if self.annotation is not None:
+            logger.info(f"plotting annotation at idx: {curr_idx} with height_ratio: {height_ratio[curr_idx]}")
+            ax_var = plt.subplot(gs[curr_idx:curr_idx + self.annotation.len(scale=annotation_scale), 0])
+            plot_annotation(
+                ax=ax_var, obj=self.annotation,
+                graph_coords=self.graph_coords,
+                plot_domain=self.annotation.add_domain,
+                distance_between_label_axis=distance_between_label_axis,
+                **self.params["annotation"])
 
             # adjust indicator lines and focus background
             set_indicator_lines(ax=ax_var, sites=self.sites, graph_coords=self.graph_coords)
             set_focus(ax=ax_var, focus=self.focus, graph_coords=self.graph_coords)
-            curr_idx += self.reference.len(scale=reference_scale)
+            curr_idx += self.annotation.len(scale=annotation_scale)
 
         if self.stroke:
+            logger.info(f"plotting stroke at idx: {curr_idx} with height_ratio: {height_ratio[curr_idx]}")
             ax_var = plt.subplot(gs[curr_idx:plots_n_rows, 0])
             plot_stroke(ax=ax_var, data=self.stroke, graph_coords=self.graph_coords, *args, **kwargs)
 
         if output:
+            logger.info(f"saving fig into {output}")
             fig.savefig(output, transparent=True, bbox_inches='tight')
         elif return_image:
             output = io.BytesIO()
@@ -1250,6 +1429,8 @@ class Plot(object):
             plt.show()
 
         plt.close()
+
+        logger.info("Plot done")
         return self
 
 
