@@ -3,11 +3,11 @@
 u"""
 This script contains the functions to draw different images
 """
+import gzip
 import math
-from collections import defaultdict
 from copy import deepcopy
 from decimal import Decimal
-from typing import Dict, List, Optional, Union
+from typing import Dict, List, Optional, Union, Tuple, Set
 
 import matplotlib as mpl
 import numpy as np
@@ -16,12 +16,13 @@ import seaborn as sns
 from adjustText import adjust_text
 from loguru import logger
 from matplotlib import pylab
+from matplotlib import pyplot as plt
 from matplotlib.font_manager import FontProperties
 from matplotlib.patches import PathPatch, Polygon
+from matplotlib.collections import PatchCollection
 from matplotlib.path import Path
 from matplotlib.textpath import TextPath
 from matplotlib.transforms import Affine2D
-from mpl_toolkits.axes_grid1.inset_locator import inset_axes
 from scipy.cluster.hierarchy import dendrogram, linkage
 from scipy.stats import gaussian_kde, zscore
 
@@ -30,11 +31,43 @@ from trackplot.base.GenomicLoci import GenomicLoci
 from trackplot.base.ReadDepth import ReadDepth
 from trackplot.base.Stroke import Stroke
 from trackplot.conf.config import CLUSTERING_METHOD, DISTANCE_METRIC
+from trackplot.file.Annotation import Annotation
 from trackplot.file.File import File
 from trackplot.file.HiCMatrixTrack import HiCTrack
 from trackplot.file.ReadSegments import ReadSegment
-from trackplot.file.Reference import Reference
-from trackplot.file.ATAC import ATAC
+
+
+def load_barcodes(barcode: str) -> Tuple[Dict[str, Dict[str, Set[str]]], Dict[str, str]]:
+    u"""
+    as name says
+    :param barcode: the path to barcode file
+    """
+    r = gzip.open(barcode, "rt") if barcode.endswith(".gz") else open(barcode, "r")
+    res = {}
+    colors = {}
+    for line in r:
+        line = line.strip().split()
+        if len(line) >= 4:
+            key, bc, group, color = line[:4]
+            colors[group] = color
+        elif len(line) == 3:
+            key, bc, group = line[:3]
+        elif len(line) == 2:
+            key, bc = line
+            group = ""
+        else:
+            continue
+
+        if key not in res.keys():
+            res[key] = {}
+
+        if group not in res[key]:
+            res[key][group] = set()
+
+        res[key][group].add(bc)
+
+    r.close()
+    return res, colors
 
 
 def get_limited_index(num, length):
@@ -114,17 +147,21 @@ def init_graph_coords(region: GenomicLoci, exons: Optional[List[List[int]]] = No
                 intron = [exons[i - 1][1], exons[i][0]]
 
                 for j in range(intron[0], intron[1]):
-                    graph_coords[j - region.start] = graph_coords[intron[0] - region.start - 1] + (
-                            j - intron[0] + 1) * intron_scale
+                    if j >= region.start:
+                        graph_coords[j - region.start] = graph_coords[intron[0] - region.start - 1] + (
+                                j - intron[0] + 1) * intron_scale
 
             for j in range(exon[0], exon[1] + 1):
-                graph_coords[j - region.start] = graph_coords[exon[0] - region.start - 1] + (
-                        j - exon[0] + 1) * exon_scale
+                if j >= region.start:
+                    graph_coords[j - region.start] = graph_coords[exon[0] - region.start - 1] + (
+                            j - exon[0] + 1) * exon_scale
 
         intron = [exons[-1][-1], region.end]
         for i in range(intron[0], intron[1]):
-            graph_coords[i - region.start] = graph_coords[intron[0] - region.start - 1] + (
-                    i - intron[0] + 1) * intron_scale
+            if i >= region.start:
+                graph_coords[i - region.start] = graph_coords[intron[0] - region.start - 1] + (
+                        i - intron[0] + 1) * intron_scale
+
     else:
         # if there is not any exons, just init graph_coords by region
         for i, j in enumerate(range(region.start, region.end + 1)):
@@ -151,7 +188,7 @@ def set_x_ticks(
         graph_coords = init_graph_coords(region)
 
     # @2018.12.19 unnecessary text in figure
-    x_label = 'Genomic coordinate (%s), "%s" strand' % (region.chromosome, region.strand)
+    x_label = str(region)
 
     if log_trans:
         if log_trans in ["2", "10"]:
@@ -195,7 +232,7 @@ def set_x_ticks(
         ax.text(x=x, y=-1, s=s, fontsize=font_size, ha="center", va="top")
 
     ax.set_ylim(-3.5, 1)
-    ax.set_xlim(0, max(graph_coords))
+    ax.set_xlim(min(graph_coords), max(graph_coords))
 
 
 def set_y_ticks(
@@ -204,7 +241,7 @@ def set_y_ticks(
         graph_coords: Union[Dict, np.array],
         max_used_y_val: Union[int, float],
         min_used_y_val: Optional[Union[int, float]] = None,
-        distance_between_label_axis: float = .1,
+        distance_between_label_axis: float = 0,
         n_y_ticks: int = 4,
         theme: str = "ticks",
         font_size: int = 5,
@@ -219,13 +256,12 @@ def set_y_ticks(
     """
     # set y ticks, y label and label
     Theme.set_theme(ax, theme)
-
+    ax.set_xlim(min(graph_coords), max(graph_coords))
     if min_used_y_val is None:
         min_used_y_val, _ = ax.get_ylim()
 
+    curr_y_tick_labels = []
     if not set_label_only:
-        ax.set_xlim(0, max(graph_coords))
-
         max_ = max_used_y_val
         plus = 0.2
         while max_ > 10:
@@ -246,7 +282,7 @@ def set_y_ticks(
         # add zero into strand-aware plot
         universal_y_ticks = np.unique(np.append(universal_y_ticks, 0))
         universal_y_ticks = sorted(universal_y_ticks)
-        curr_y_tick_labels = []
+
         for lab in universal_y_ticks:
             if y_axis_skip_zero and lab == 0:
                 # Exclude label for 0
@@ -268,9 +304,19 @@ def set_y_ticks(
     @2018.12.20 using BAM label as y label
     @2019.01.04 change the standards of distance between y label and y-axis
     """
+
     if show_y_label:
+        def __dynamic_distance__(distance_between_label_axis: float, label: str, scale: int = 100) -> float:
+            if distance_between_label_axis != 0:
+                return -distance_between_label_axis
+
+            return max(0.01, math.ceil(len(label) / 10) * 10 / scale) * -1
+
+        curr_y_tick_labels = sorted(curr_y_tick_labels, key=lambda x: len(x), reverse=True)
         ax.text(
-            x=-1 * distance_between_label_axis * max(graph_coords),
+            x=__dynamic_distance__(
+                distance_between_label_axis, curr_y_tick_labels[0] if curr_y_tick_labels else ""
+            ) * max(graph_coords),
             y=(max_used_y_val + min_used_y_val) / 2,
             s=label, fontsize=font_size, ha="right"
         )
@@ -293,7 +339,7 @@ def set_focus(
             fill_y = [y1, y1, y2, y2]
             ax.fill(fill_x, fill_y, alpha=0.1, color='grey')
         except IndexError as err:
-            logger.warning("focus region is out of bound: " + str(err))
+            logger.debug("focus region is out of bound: " + str(err))
 
 
 def set_indicator_lines(
@@ -320,7 +366,7 @@ def set_indicator_lines(
                 lw=0.5
             )
         except IndexError as err:
-            logger.warning("Indicator line is out of bound: " + str(err))
+            logger.debug("Indicator line is out of bound: " + str(err))
 
 
 def plot_stroke(
@@ -328,7 +374,6 @@ def plot_stroke(
         data: List[Stroke],
         graph_coords: Optional[Union[Dict, np.ndarray]] = None,
         font_size: int = 5,
-        distance_between_label_axis: Union[int, float] = .1,
         theme: str = "blank",
         **kwargs
 ):
@@ -340,36 +385,23 @@ def plot_stroke(
 
     for i, stroke in enumerate(strokes):
         try:
-            ax.hlines(
-                y=i,
-                xmin=graph_coords[stroke.start],
-                xmax=graph_coords[stroke.end],
-                color=stroke.color, lw=2)
-            ax.text(
-                -1 * distance_between_label_axis * max(graph_coords),
-                i + .2,
-                stroke.label,
-                fontsize=font_size,
-                color=stroke.color
-            )
-        except IndexError as err:
-            logger.warning(f"stroke is out of bound: {err}")
+            ax.hlines(y=i, xmin=graph_coords[stroke.start], xmax=graph_coords[stroke.end],  color=stroke.color, lw=2)
+            ax.text(-.1 * max(graph_coords),  i + .2, stroke.label, fontsize=font_size, color=stroke.color)
+        except IndexError as _:
+            logger.info(f"stroke {stroke} is out of bound, this stroke won't show up in final plot")
 
     Theme.set_theme(ax, theme)
-    ax.set_xlim(left=0, right=max(graph_coords))
+    ax.set_xlim(min(graph_coords), max(graph_coords))
     ax.set_ylim(bottom=-1, top=len(strokes))
 
 
-def plot_reference(
+def plot_annotation(
         ax: mpl.axes.Axes,
-        obj: Reference,
+        obj: Annotation,
         graph_coords: Optional[Union[Dict, np.ndarray]] = None,
         font_size: int = 5,
         show_gene: bool = False,
         show_id: bool = False,
-        transcripts: Optional[List[str]] = None,
-        remove_empty_transcripts: bool = False,
-        choose_primary: bool = False,
         color: Optional[str] = None,
         theme: str = "blank",
         y_loc: int = 0,
@@ -377,14 +409,13 @@ def plot_reference(
         plot_domain: bool = False,
         show_exon_id: bool = False,
         raster: bool = True,
-        distance_between_label_axis: float = 0.3,
         **kwargs
 ):
     u"""
-    Plot the structure of reference
+    Plot the structure of annotation
     """
     Theme.set_theme(ax, theme)
-    ax.set_xlim(0, max(graph_coords))
+    ax.set_xlim(min(graph_coords), max(graph_coords))
 
     region = obj.region
 
@@ -397,77 +428,50 @@ def plot_reference(
 
     color = "k" if not color else color
 
-    if choose_primary and (len(transcripts) == 0 or transcripts is None):
-        transcripts = []
-        # For each gene, you can choose only one transcript to plot.
-        genes = defaultdict(list)
-        for transcript in data:
-            if transcript.category == 'interval':
-                continue
-            genes[transcript.gene_id].append(transcript)
-
-        for _, transcripts_list in genes.items():
-            primary_transcripts = sorted(
-                transcripts_list, key=lambda i_: len(i_), reverse=True)[0]
-            transcripts.append(primary_transcripts.transcript_id)
-    elif choose_primary and (len(transcripts) != 0 or transcripts is not None):
-        logger.warning(
-            "--transcripts-to-show is prior to --choose-primary, and primary transcript won't be presented.")
-    else:
-        pass
-
     """
     @2018.12.26
     Maybe I'm too stupid for this, using 30% of total length of x axis as the gap between text with axis
     """
-
-    for transcript in obj.data:
-        # ignore the unwanted transcript
-        if transcripts and not (set(transcripts) & set(transcript.ids())):
-            continue
-
-        # ignore transcripts without any exons
-        if remove_empty_transcripts and not transcript.exons:
-            continue
-
+    patches, arrows = [], {}
+    for transcript in data:
         strand = transcript.strand
         # @2018.12.20 add transcript id, based on fixed coordinates
         if transcript.transcript:
             if show_gene and transcript.gene and transcript.gene_id != transcript.transcript_id:
                 if show_id:
-                    ax.text(x=-1 * distance_between_label_axis * max(graph_coords), y=y_loc + 0.25,
+                    ax.text(x=-.01 * max(graph_coords), y=y_loc + 0.25,
                             s=transcript.gene_id, fontsize=font_size, ha="right")
-                    ax.text(x=-1 * distance_between_label_axis * max(graph_coords), y=y_loc - 0.25,
+                    ax.text(x=-.01 * max(graph_coords), y=y_loc - 0.25,
                             s=transcript.transcript_id, fontsize=font_size, ha="right")
                 else:
-                    ax.text(x=-1 * distance_between_label_axis * max(graph_coords), y=y_loc,
-                            s=transcript.gene + " | " + transcript.transcript,
-                            fontsize=font_size, ha="right")
+                    ax.text(x=-.01 * max(graph_coords), y=y_loc,
+                            s=transcript.gene + " | " + transcript.transcript, fontsize=font_size, ha="right")
             else:
-                ax.text(x=-1 * distance_between_label_axis * max(graph_coords), y=y_loc - 0.1, s=transcript.transcript,
-                        fontsize=font_size, ha="right")
+                ax.text(x=-1, y=y_loc - 0.1, s=transcript.transcript, fontsize=font_size, ha="right")
 
         # @2018.12.19
         # s and e is the start and end site of single exon
         # @2022.05.13
         # add index to avoid label overlapping of neighbor exon
         for ind, exon in enumerate(transcript.exons):
-            s, e, strand = region.relative(
-                exon.start), region.relative(exon.end), exon.strand
-            x = [
-                graph_coords[s], graph_coords[e],
-                graph_coords[e], graph_coords[s]
-            ]
-            y = [
-                y_loc - exon_width / 2, y_loc - exon_width / 2,
-                y_loc + exon_width / 2, y_loc + exon_width / 2
-            ]
-            ax.fill(x, y, color, lw=.5, zorder=20)
+            s, e, strand = (region.relative(exon.start),
+                            region.relative(exon.end), exon.strand)
 
-            if show_exon_id:
+            if s < 0 or e < 0:
+                continue
+
+            patches.append(plt.Rectangle(
+                (graph_coords[s], y_loc - exon_width / 2),
+                width=graph_coords[e]-graph_coords[s], height=exon_width,
+                lw=.5, zorder=20,rasterized=raster, color=color
+            ))
+
+            if show_exon_id and exon.name:
                 y_loc_offset = 0.1 if ind % 2 == 0 else - 0.2
-                ax.text(x=(graph_coords[s] + graph_coords[s]) / 2, y=y_loc + y_loc_offset,
-                        s=exon.name, fontsize=font_size / 2, ha="right")
+                ax.text(x=(graph_coords[s] + graph_coords[e]) / 2,
+                        y=y_loc + y_loc_offset,
+                        s=exon.name, fontsize=font_size / 2,
+                        ha="center", color="red")
 
         # @2018.12.21
         # change the intron range
@@ -492,15 +496,12 @@ def plot_reference(
 
                 for i in range(narrows):
                     loc = float(i) * length / narrows + graph_coords[region.relative(transcript.start)]
-                    if strand == '+':
-                        x = [loc - spread, loc, loc - spread]
-                    else:
-                        x = [loc + spread, loc, loc + spread]
-                    y = [y_loc - exon_width / 5, y_loc, y_loc + exon_width / 5]
-                    ax.plot(x, y, lw=.5, color=color, rasterized=raster)
+
+                    arrows[(loc - spread if strand == "+" else loc + spread, y_loc - exon_width / 5)] = Path.MOVETO
+                    arrows[(loc, y_loc)] = Path.LINETO
+                    arrows[(loc - spread if strand == "+" else loc + spread, y_loc + exon_width / 5)] = Path.LINETO
 
         y_loc += 1  # if transcript.transcript else .5
-
         if plot_domain and obj.domain and transcript.transcript_id in obj.domain.pep:
             current_domains = obj.domain.pep[transcript.transcript_id]
 
@@ -519,16 +520,11 @@ def plot_reference(
                         s = 0 if s < 0 else s
                         e = len(region) - 1 if e > len(region) else e
 
-                        x = [
-                            graph_coords[s], graph_coords[e],
-                            graph_coords[e], graph_coords[s]
-                        ]
-                        y = [
-                            y_loc - exon_width / 4, y_loc - exon_width / 4,
-                            y_loc + exon_width / 4, y_loc + exon_width / 4
-                        ]
-                        ax.fill(x, y, color, lw=.5,
-                                zorder=20, rasterized=raster)
+                        patches.append(plt.Rectangle(
+                            (graph_coords[s], y_loc - exon_width / 2),
+                            width=graph_coords[e] - graph_coords[s], height=exon_width,
+                            lw=.5, zorder=20,rasterized=raster, color=color
+                        ))
 
                     # @2022.05.13
                     intron_relative_s = region.relative(
@@ -544,11 +540,12 @@ def plot_reference(
                     if intron_relative_e <= 0:
                         continue
 
-                    intron_sites = [graph_coords[intron_relative_s],
-                                    graph_coords[intron_relative_e]]
                     if len(sub_exon) != 1:
-                        ax.plot(intron_sites, [y_loc, y_loc],
-                                color=color, lw=0.2, rasterized=raster)
+                        patches.append(plt.Rectangle(
+                            (graph_coords[intron_relative_s], y_loc),
+                            width=graph_coords[intron_relative_e] - graph_coords[intron_relative_s],
+                            height=0, color=color, lw=0.2, rasterized=raster
+                        ))
                 if show_id:
                     ax.text(x=-1, y=y_loc - 0.125, s=f"{sub_current_domain.gene}|{transcript.transcript_id}",
                             fontsize=font_size / 2, ha="right")
@@ -556,10 +553,8 @@ def plot_reference(
                     ax.text(x=-1, y=y_loc - 0.125, s=f"{sub_current_domain.gene}|{transcript.transcript}",
                             fontsize=font_size / 2, ha="right")
 
-                y_loc += 0.25
+                y_loc += .5
 
-        # offset for next term.
-        y_loc += 0.75
     if obj.local_domain:
         for base_name, current_domain in obj.local_domain.items():
             for sub_current_domain in current_domain:
@@ -577,16 +572,11 @@ def plot_reference(
                         s = 0 if s < 0 else s
                         e = len(region) - 1 if e > len(region) else e
 
-                        x = [
-                            graph_coords[s], graph_coords[e],
-                            graph_coords[e], graph_coords[s]
-                        ]
-                        y = [
-                            y_loc - exon_width / 4, y_loc - exon_width / 4,
-                            y_loc + exon_width / 4, y_loc + exon_width / 4
-                        ]
-                        ax.fill(x, y, color, lw=.5,
-                                zorder=20, rasterized=raster)
+                        patches.append(plt.Rectangle(
+                            (graph_coords[s], y_loc - exon_width / 2),
+                            width=graph_coords[e] - graph_coords[s], height=exon_width,
+                            lw=.5, zorder=20, rasterized=raster, color=color
+                        ))
 
                     # @2022.05.13
                     intron_relative_s = region.relative(
@@ -602,18 +592,23 @@ def plot_reference(
                     if intron_relative_e <= 0:
                         continue
 
-                    intron_sites = [graph_coords[intron_relative_s],
-                                    graph_coords[intron_relative_e]]
                     if len(sub_exon) != 1:
-                        ax.plot(intron_sites, [y_loc, y_loc],
-                                color=color, lw=0.2, rasterized=raster)
+                        patches.append(plt.Rectangle(
+                            (graph_coords[intron_relative_s], y_loc),
+                            width=graph_coords[intron_relative_e]-graph_coords[intron_relative_s],
+                            height=0, color=color, lw=0.2, rasterized=raster
+                        ))
 
                 ax.text(x=-1, y=y_loc - 0.125, s=f"{sub_current_domain.gene}|{base_name}",
                         fontsize=font_size / 2, ha="right")
 
             y_loc += 1
 
+    ax.add_collection(PatchCollection(patches, match_original=True))
+    ax.add_patch(PathPatch(Path(list(arrows.keys()), list(arrows.values())), lw=.5, color=color, rasterized=raster))
+
     # @2022.05.13 Set y lim using y_loc value.
+    ax.set_xlim(min(graph_coords), max(graph_coords))
     ax.set_ylim(-.5, y_loc + .5)
 
 
@@ -675,44 +670,48 @@ def plot_density(
 
     if data is None:
         data = obj.data
-    jxns = data.junctions_dict
+
+    try:
+        jxns = data.junctions_dict
+    except AttributeError:
+        # depth do not have junctions
+        jxns = {}
 
     fixed_max_used_y, fixed_min_used_y = max_used_y_val is not None, min_used_y_val is not None
     if max_used_y_val is None:
-        max_used_y_val = max(data.plus)
+        if isinstance(data, dict):
+            for k, v in data.items():
+                max_used_y_val = max(v.plus.max(), max_used_y_val if max_used_y_val else 0)
+        else:
+            max_used_y_val = max(data.plus)
         if max_used_y_val % 2 == 1:
             max_used_y_val += 1
 
     if min_used_y_val is None:
-        min_used_y_val = -1 * max(data.minus) if data.minus is not None else 0
+        if isinstance(data, dict):
+            min_used_y_val = [v.minus.max() if v.minus else 0 for v in data.values()]
+            min_used_y_val = -1 * max(min_used_y_val)
+        else:
+            min_used_y_val = -1 * max(data.minus) if data.minus is not None else 0
 
     # Reduce memory footprint by using incremented graph_coords.
     x, y1, y2 = [], [], []
     for i in range(len(graph_coords)):
         x.append(graph_coords[i])
-        y1.append(data.plus[i] if data.plus is not None else 0)
-        y2.append(-data.minus[i] if data.minus is not None else 0)
+
+        if isinstance(data, dict):
+            y1.append(max([v.plus[i] if v.plus is not None else 0 for v in data.values()]))
+            y2.append(min([-v.minus[i] if v.minus is not None else 0 for v in data.values()]))
+        else:
+            y1.append(data.plus[i] if data.plus is not None else 0)
+            y2.append(-data.minus[i] if data.minus is not None else 0)
 
     ax.fill_between(x, y1, y2=y2, color=color, lw=0, step=fill_step, rasterized=raster)
 
-    # if isinstance(obj, ATAC):
-    #     for idx, y in enumerate([y1, y2]):
-    #         if np.sum(y) > 0:
-    #             array_hist = np.repeat(graph_coords, np.abs(y1).astype(np.int32))
-    #             try:
-    #                 kde = gaussian_kde(array_hist)
-    #                 fit_value = kde.pdf(graph_coords)
-    #             except (ValueError, np.linalg.LinAlgError):
-    #                 # logger.warning(err)
-    #                 # logger.warning(traceback.format_exc())
-    #                 continue
-    #
-    #             fit_value = fit_value / fit_value.max()
-    #             ax.plot(graph_coords, fit_value * np.max(y), c=color, lw=1)
-
-    if data.strand_aware:
-        max_used_y_val = max(abs(min_used_y_val), max_used_y_val)
-        min_used_y_val = -max(abs(min_used_y_val), max_used_y_val) if data.minus is not None else 0
+    if not isinstance(data, dict):
+        if data.strand_aware:
+            max_used_y_val = max(abs(min_used_y_val), max_used_y_val)
+            min_used_y_val = -max(abs(min_used_y_val), max_used_y_val) if data.minus is not None else 0
 
     if jxns:
         # sort the junctions by intron length for better plotting look
@@ -815,10 +814,9 @@ def plot_density(
                 jxn_numbers.append(t)
 
         try:
-            adjust_text(jxn_numbers, force_text=0.2, arrowprops=dict(arrowstyle="-", color='black', lw=1),
-                        autoalign="y")
+            adjust_text(jxn_numbers, force_text=(0.2, 0.2), arrowprops=dict(arrowstyle="-", color='black', lw=1))
         except IndexError as err:
-            logger.warning(err)
+            logger.debug(err)
 
     if obj and obj.title:
         ax.text(max(graph_coords) - len(obj.title), max_used_y_val, obj.title, color=color, fontsize=font_size)
@@ -833,7 +831,7 @@ def plot_density(
     if max_used_y_val == min_used_y_val == 0:
         min_used_y_val, max_used_y_val = ax.get_ylim()
 
-    if data.strand_aware and kwargs.get("density_by_strand"):
+    if not isinstance(data, dict) and data.strand_aware and kwargs.get("density_by_strand"):
         max_used_y_val = max(abs(min_used_y_val), max_used_y_val)
         min_used_y_val = -max_used_y_val
     elif not kwargs.get("density_by_strand") and not jxns:
@@ -849,7 +847,7 @@ def plot_density(
         distance_between_label_axis=distance_between_label_axis,
         font_size=font_size,
         show_y_label=show_y_label,
-        y_axis_skip_zero=False if data.strand_aware else True
+        y_axis_skip_zero=False if not isinstance(data, dict) and data.strand_aware else True
     )
 
 
@@ -897,6 +895,7 @@ def plot_site_plot(
     max_height, min_height = max(plus), min(minus)
     max_val = max(max_height, abs(min_height))
 
+    patches = []
     for label, array_plot in zip(['plus', 'minus'], [plus, minus]):
         if strand_choice != "all" and label != strand_choice:
             continue
@@ -906,16 +905,19 @@ def plot_site_plot(
             kde = gaussian_kde(array_hist)
             fit_value = kde.pdf(graph_coords)
         except (ValueError, np.linalg.LinAlgError):
-            # logger.warning(err)
-            # logger.warning(traceback.format_exc())
+            # logger.debug(err)
+            # logger.debug(traceback.format_exc())
             continue
 
         fit_value = fit_value / fit_value.max()
-        ax.bar(graph_coords, array_plot, color=color, rasterized=raster)
+
+        for x, y in zip(graph_coords, array_plot):
+            patches.append(plt.Rectangle((x, 0), height=y, width=.6, color=color, rasterized=raster))
         ax.plot(graph_coords,
                 fit_value * array_plot.max() if label == 'plus' else fit_value * array_plot.min(),
                 c=color, lw=1)
 
+    ax.add_collection(PatchCollection(patches, match_original=True))
     # set the y limit
     # set y ticks, y label and label
     set_y_ticks(
@@ -1000,6 +1002,10 @@ def plot_heatmap(
     if not show_row_names:
         labels = False
 
+    if vmin == vmax == 0 and (mtx.max() != 0 or mtx.min() != 0):
+        logger.debug("The vmin and vmax == 0, but input matrix not != 0")
+        vmin, vmax = None, None
+
     sns.heatmap(mtx, ax=ax, cmap=color, cbar_ax=cbar_ax,
                 xticklabels=False, yticklabels=labels,
                 center=False, rasterized=raster, vmin=vmin, vmax=vmax)
@@ -1161,7 +1167,7 @@ def plot_line(
         try:
             adjust_text(legend, arrowprops=dict(arrowstyle="-", lw=1))
         except IndexError as err:
-            logger.warning(err)
+            logger.debug(err)
 
     set_y_ticks(
         ax, label=y_label, theme=theme,
@@ -1194,6 +1200,7 @@ def plot_igv_like(
     assert len(obj) == 1, "IGV-like plot only support one file"
 
     obj = list(obj.values())[0]
+    assert isinstance(obj, ReadSegment), "the input obj should be ReadSegment"
     assert obj.region is not None, "please load data first"
 
     region = obj.region
@@ -1204,6 +1211,7 @@ def plot_igv_like(
         y_label = obj.label
     y_loc = 0.5
 
+    patches, scatters_x, scatters_y = [], [], []
     # Add this to skip zero coverage regions.
     if obj.meta is not None:
 
@@ -1224,17 +1232,12 @@ def plot_igv_like(
                     s = 0 if s < 0 else s
                     e = len(region) - 1 if e >= len(region) else e
 
-                    x = [
-                        graph_coords[s], graph_coords[e],
-                        graph_coords[e], graph_coords[s]
-                    ]
-                    y = [
-                        y_loc - exon_width, y_loc - exon_width,
-                        y_loc + exon_width, y_loc + exon_width
-                    ]
+                    patches.append(plt.Rectangle((graph_coords[s], y_loc - exon_width),
+                                             width=graph_coords[e] - graph_coords[s],
+                                             height=exon_width * 2,
+                                             facecolor='k' if not exon_color else exon_color,
+                                             lw=.5, zorder=20))
 
-                    ax.fill(x, y, 'k' if not exon_color else exon_color,
-                            lw=.5, zorder=20)
                     add_plot = add_plot | True
 
                 for intron in c_data.introns:
@@ -1249,9 +1252,12 @@ def plot_igv_like(
                         graph_coords[s],
                         graph_coords[e]
                     ]
-                    ax.plot(intron_sites, [y_loc, y_loc],
-                            color="#4d4d4d" if not intron_color else intron_color,
-                            lw=0.2, rasterized=raster)
+                    patches.append(plt.Rectangle(
+                        (graph_coords[s], y_loc),
+                        width=graph_coords[e]-graph_coords[s], height=0,
+                        color="#4d4d4d" if not intron_color else intron_color,
+                        lw=.2, rasterized=raster
+                    ))
 
                 for feature in c_data.features:
                     if feature.start == -1:
@@ -1271,29 +1277,33 @@ def plot_igv_like(
                     s = 0 if s < 0 else s
                     e = len(region) - 1 if e >= len(region) else e
 
-                    x = [
-                        graph_coords[s], graph_coords[e],
-                        graph_coords[e], graph_coords[s]
-                    ]
                     width_ratio = 0.75 if not is_site else 1.5
 
-                    y = [
-                        y_loc - exon_width * width_ratio, y_loc - exon_width * width_ratio,
-                        y_loc + exon_width * width_ratio, y_loc + exon_width * width_ratio
-                    ]
-                    ax.fill(
-                        x, y, 'r' if not feature_color else feature_color, lw=.2, zorder=20)
                     if is_site:
-                        ax.scatter(graph_coords[s],
-                                   y_loc + exon_width * width_ratio,
-                                   c='b' if not feature_color else feature_color,
-                                   s=.5,
-                                   linewidths=(0,),
-                                   rasterized=raster
-                                   )
+                        patches.append(plt.Rectangle((graph_coords[s], y_loc - exon_width * width_ratio),
+                                                 width=graph_coords[e] - graph_coords[s],
+                                                 height=exon_width * 2,
+                                                 facecolor='blue' if not feature_color else feature_color,
+                                                 lw=.2, zorder=20, rasterized = raster))
+
+                        scatters_x.append(graph_coords[s])
+                        scatters_y.append(y_loc + exon_width * width_ratio)
+                    else:
+                        patches.append(plt.Rectangle((graph_coords[s], y_loc - exon_width * width_ratio),
+                                                 width=graph_coords[e] - graph_coords[s],
+                                                 height=exon_width * 2,
+                                                 facecolor='red' if not feature_color else feature_color,
+                                                 lw=.2, zorder=20, rasterized = raster))
             if add_plot:
                 y_loc += 1
 
+    # using patches and draw scatter in bath to reduce time
+    ax.add_collection(PatchCollection(patches, match_original=True))
+    ax.scatter(
+        scatters_x, scatters_y, rasterized=raster,
+        color='b' if not feature_color else feature_color, s=.5, linewidths=(0,)
+    )
+
     set_y_ticks(
         ax, label=y_label, theme=theme,
         graph_coords=graph_coords,
@@ -1323,7 +1333,7 @@ def plot_links(ax: mpl.axes.Axes,
         a = Path(pts, [Path.MOVETO, Path.CURVE4, Path.CURVE4, Path.CURVE4])
         ax.add_patch(PathPatch(a, fc="none", ec=stroke.color, lw=1))
 
-    ax.set_xlim(0, max(graph_coords))
+    ax.set_xlim(min(graph_coords), max(graph_coords))
     ax.set_ylim(max_y, 0)
     ax.axis("off")
 
@@ -1362,6 +1372,9 @@ def plot_motif(ax: mpl.axes.Axes,
         graph_coords = init_graph_coords(region)
 
     # 在原始坐标轴上画motif
+    if not data:
+        logger.info("there is no any motif information to plot")
+        return
     ymin, ymax, xmin, xmax = 0, 0, \
                              graph_coords[min(data.keys()) - region.start], \
                              graph_coords[max(data.keys()) - region.start] + (1 + width) / 2
@@ -1374,10 +1387,8 @@ def plot_motif(ax: mpl.axes.Axes,
         bbox_to_left = max(xmin / len(graph_coords) - axin_width - .1, 0)
         draw_left = True
 
-    axins = inset_axes(ax, width=f"{axin_width * 100}%", height="100%",
-                       loc='center left',
-                       bbox_to_anchor=(bbox_to_left, 0, 1, 1),
-                       bbox_transform=ax.transAxes)
+    # update insert_axes to matplotlib 3.7.2
+    axins = ax.inset_axes(bounds=[bbox_to_left, 0, axin_width, 1], transform=ax.transAxes)
 
     start_site = min(list(data.keys()))
     site = 0
